package eu.mhsl.craftattack.spawn.varo.appliances.metaGameplay.joinProtection;

import eu.mhsl.craftattack.core.Main;
import eu.mhsl.craftattack.core.appliance.Appliance;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import net.kyori.adventure.util.Ticks;
import org.bukkit.Bukkit;
import org.bukkit.entity.Player;
import org.bukkit.event.Cancellable;
import org.bukkit.event.Listener;
import org.bukkit.potion.PotionEffect;
import org.bukkit.potion.PotionEffectType;
import org.jetbrains.annotations.NotNull;
import org.jetbrains.annotations.Nullable;

import java.util.*;

public class JoinProtection extends Appliance {
    private static final int resistanceDuration = 10;

    static final class Options {
        public boolean wasOnGround = false;
        public Long joinTime = System.currentTimeMillis();
    }
    private final Map<UUID, Options> protectedPlayers = new HashMap<>();

    public void addProtection(Player player) {
        this.protectedPlayers.put(player.getUniqueId(), new Options());
        PotionEffect resistance = new PotionEffect(PotionEffectType.RESISTANCE, Ticks.TICKS_PER_SECOND * resistanceDuration, 1);
        PotionEffect blindness = new PotionEffect(PotionEffectType.DARKNESS, Ticks.TICKS_PER_SECOND * 3, 1);
        player.addPotionEffects(List.of(resistance, blindness));

        Bukkit.getScheduler().runTaskLater(
            Main.instance(),
            () -> this.protectedPlayers.remove(player.getUniqueId()),
            Ticks.TICKS_PER_SECOND * resistanceDuration
        );
    }

    @Nullable JoinProtection.Options getOptions(Player player) {
        return this.protectedPlayers.get(player.getUniqueId());
    }

    public void cancelEvent(Player player, Cancellable event) {
        if(!this.protectedPlayers.containsKey(player.getUniqueId())) return;
        event.setCancelled(true);

        Options option = this.protectedPlayers.get(player.getUniqueId());
        int secondsLeft = Math.abs((int) ((System.currentTimeMillis() - option.joinTime) / 1000) - resistanceDuration);
        player.sendActionBar(
            Component.text(
                String.format("Du bist in %d Sekunden angreifbar", secondsLeft),
                NamedTextColor.RED
            )
        );
    }

    @Override
    protected @NotNull List<Listener> listeners() {
        return List.of(new JoinProtectionListener());
    }
}
